package com.brilliance.isc.funds.crtcri.service;

import cn.hutool.core.bean.BeanUtil;
import com.brilliance.isc.bo.*;
import com.brilliance.isc.bo.model.Crdgrp;
import com.brilliance.isc.bo.model.Crtp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cache.pty.PtyCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.nosmod.NosmodService;
import com.brilliance.isc.common.ordmod.service.OrdmodService;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.CrtcriStoreBo;
import com.brilliance.isc.funds.common.component.CrToolComponent;
import com.brilliance.isc.funds.crtcri.check.CrtcriValidator;
import com.brilliance.isc.mda.dao.*;
import com.brilliance.isc.vo.funds.CrdBizInfoVo;
import com.brilliance.mda.runtime.mda.CodetableItem;
import com.brilliance.mda.runtime.mda.IStream;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.impl.StreamImpl;
import com.brilliance.mda.runtime.mda.util.*;
import com.google.common.collect.ImmutableBiMap;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: CrtcriTransactionServiceImpl
 * @description: 汇入清算 Crtcri 业务处理
 * @author: huangshunlin
 * @create: 2024-10-16 22:35
 */
@Service("crtcri.transaction")
public class CrtcriTransactionServiceImpl extends AbstractTransactionService<CrtcriStoreBo> {


    private static final Logger logger = LoggerFactory.getLogger(CrtcriTransactionServiceImpl.class);


    @Resource
    private CrtcriValidator crtcriValidator;


    @Resource
    private SysmodService sysmodService;

    @Resource
    private CrToolComponent crToolComponent;

    @Resource
    private TransactionHelpService transactionHelpService;

    @Resource
    private SettleContext settleContext;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private TrnmodService trnmodService;


    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;


    @Resource
    private ActMapper actMapper;

    @Resource
    private SmhMapper smhMapper;


    @Resource
    private PtyMapper ptyMapper;

    @Resource
    private PtaMapper ptaMapper;

    @Resource
    private PtsptaToolComponent ptsptaToolComponent;

    @Resource
    private BchMapper bchMapper;

    @Resource
    private TrnMapper trnMapper;

    @Resource
    private OrdmodService ordmodService;

    @Resource
    private PublicMapper publicMapper;

    @Resource
    private NosmodService nosmodService;

    @Resource
    private PtyCacheService ptyCacheService;

    @Resource
    private CrtpService crtpService;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_JSH_SERVICE, "Y");
    }


    @Override
    protected void txnSave(CrtcriStoreBo transactionVo) {
        defaultCrdgrpRecNamN1000(transactionVo);
        ruleMtabutSavN500(transactionVo);
    }

    /**
     * source:crtcri.0013.script
     */
    public void ruleMtabutSavN500(CrtcriStoreBo bo) {
        Crdgrp crdgrp = bo.getCrdgrp();
        crToolComponent.getref(crdgrp);
        // 1. Create (rsp. update) Contract (incl. Texts)
        // change state
        bo.getCrdgrp().getRec().setSta("D");
        crToolComponent.sav(crdgrp);
        // 2. Build Transaction from CRDGRP
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();
        trnmodService.trnSetFromContract(trnmodVo);
        cbsSav(bo);
        if (MdaUtils.compareTo(bo.getCrdgrp().getRec().getTrntyp(), "M") == 0) {
            // 手工清算，登记SPT后续交易
            manualClear(bo);
        }
        String sql = NULLSTR;
        if (sysmodService.sptIsAnyLoaded(bo.getSpt().getSta()) && !sysmodService.sptIsIncLoaded(bo.getSpt().getSta())) {
            // 处理Repair后Take，修改对公/对私处理的情况
            if (!MdaUtils.isEmpty(bo.getCrdgrp().getRec().getSptinr())) {
                if (MdaUtils.compareTo(bo.getCrdgrp().getRec().getCustyp(), "0") == 0) {
                    // 对公
                    sql = "UPDATE SPT SET DELFLG = ' ' WHERE INR = '" + bo.getCrdgrp().getRec().getSptinr() + "'";
                } else {
                    // 对私
                    sql = "UPDATE SPT SET DELFLG = 'X' WHERE INR = '" + bo.getCrdgrp().getRec().getSptinr() + "'";
                }
                int count = publicMapper.updateByTableNameAndInr(ImmutableBiMap.of("sql", sql));
                logger.info("update spt count ={}", count);
            }
        }
        SetmodVo setmodVo = transactionHelpService.getTransactionStoreSet().getSetmodVo();
        nosmodService.savNostro(bo.getNosmod(), setmodVo, trnmodVo);

    }

    /**
     * source:crtcri.0012.script
     */
    private void cbsSav(CrtcriStoreBo bo) {
        String trninr = transactionHelpService.getTrninr();
        cbsmodService.cbsTrnSetNew("TRN", trninr);
        trnmodService.cbsTrnStore("MAXAMT", "AMT1", Dates.today(), bo.getCrdgrp().getCbs().getMax().getCur()
                , bo.getCrdgrp().getCbs().getMax().getAmt(), I18nUtil.getMessage("funds_crtcri_000000"),
                bo.getCrdgrp().getRec(), trninr);
    }

    @Override
    protected Map<String, String> txnCheck(CrtcriStoreBo transactionVo) {
        return crtcriValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(CrtcriStoreBo transactionVo) {
        Crd rec = transactionVo.getCrdgrp().getRec();
        Crtp crtp = transactionVo.getCrtp();
        CrdBizInfoVo bizInfoVo = new CrdBizInfoVo();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setObjtyp("CRD");
        bizInfoVo.setForare(crtp.getForare());
        bizInfoVo.setOpndat(rec.getOpndat());
        bizInfoVo.setRcvbchinr(rec.getRcvbchinr());
        bizInfoVo.setCustyp(rec.getCustyp());
        bizInfoVo.setRcvactInr(transactionVo.getRcvact().getInr());
        bizInfoVo.setRcvactExtkey(transactionVo.getRcvact().getExtkey());
        bizInfoVo.setRcvobjtyp(rec.getRcvobjtyp());
        Cpd cpd = transactionVo.getCpdgrp().getRec();
        bizInfoVo.setValdat(cpd.getValdat());

        bizInfoVo.setBranchInr(rec.getBranchinr());
        bizInfoVo.setBchkeyinr(rec.getBchkeyinr());
        bizInfoVo.setOwnref(rec.getOwnref());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(CrtcriStoreBo transactionVo) {
        return null;
    }

    @Override
    public CrtcriStoreBo init(CrtcriStoreBo crtcriStoreBo) {
        crtcriStoreBo.reset();
        init950(crtcriStoreBo);
        init1000(crtcriStoreBo);
        return crtcriStoreBo;
    }

    /**
     * source:crtcri.0001.script
     */
    public void init1000(CrtcriStoreBo bo) {
        // 不让Break
//        Systems.setValues(bo.getCrdgrp().getRec(), "rcvobjtyp", "CPD" + CR + "BOD" + CR + "CCD" + CR + "CLD" + CR + "BED" + CR + "BTD" + CR + "GID");
        bo.getCrdgrp().getRec().setErrmsg(NULLSTR);
        if (sysmodService.sptIsIncLoaded(bo.getSpt().getSta())) {
            bo.getCrdgrp().getCbs().getMax().setCur(bo.getSpt().getRelcur());
            bo.getCrdgrp().getCbs().getMax().setAmt(bo.getSpt().getRelamt());
            bo.getCrdgrp().getRec().setMsgact(bo.getSpt().getAct59());
            bo.getCrdgrp().getRec().setMsgref(bo.getSpt().getObjref());
            bo.getCrdgrp().getRec().setClrtyp("I");
            bo.getCrdgrp().getRec().setTrntyp("A");
            bo.getCrdgrp().getRec().setCredat(Dates.today());
            bo.getCrdgrp().getRec().setOpndat(Dates.today());
            bo.getCrdgrp().getRec().setSptinr(bo.getSpt().getInr());
            Smh querySmh = new Smh();
            querySmh.setInr(bo.getSpt().getSmhinr());
            Smh smh = smhMapper.selectByPrimaryKey(querySmh);
            if (Objects.isNull(smh)) {
                bo.getCrdgrp().getRec().setSta("E");
                bo.getCrdgrp().getRec().setErrmsg("根据主键" + bo.getSpt().getSmhinr() + "未找到表记录");
            } else {
                bo.setSmh(smh);
                bo.getCrdgrp().getRec().setRcvbic(bo.getSmh().getRcvkey());
                bo.getCrdgrp().getRec().setMsgtyp(bo.getSmh().getMsgtyp());
                bo.getCrdgrp().getRec().setMsginr(bo.getSmh().getInr());
                if (!MdaUtils.isEmpty(bo.getSmh().getGrpinr())) {
                    String sql = "select * from smh WHERE GRPINR = '" + bo.getSmh().getGrpinr() + "' AND INR <> '" + bo.getSmh().getInr() + "'";
                    List<Smh> smhList = smhMapper.dyncRead(ImmutableBiMap.of("sql", sql));
                    if (CollectionUtils.isEmpty(smhList)) {
                        bo.getCrdgrp().getRec().setSta("E");
                        bo.getCrdgrp().getRec().setErrmsg("根据GRPINR" + bo.getSpt().getSmhinr() + "未找到表记录");
                    } else {
                        bo.setCorsmh(smhList.get(0));
                        bo.getCrdgrp().getRec().setBndtyp(bo.getCorsmh().getMsgtyp());
                        bo.getCrdgrp().getRec().setBndinr(bo.getCorsmh().getInr());
                        if (!MdaUtils.isEmpty(bo.getCorsmh().getRelcur()) && !MdaUtils.isEmpty(bo.getCorsmh().getRelamt())) {
                            bo.getCrdgrp().getCbs().getMax().setCur(bo.getCorsmh().getRelcur());
                            bo.getCrdgrp().getCbs().getMax().setAmt(bo.getCorsmh().getRelamt());
                        }
                    }
                } else {
//                    MdaUtils.clear(bo.getCorsmh());
                    bo.setCorsmh(new Smh());
                }
            }
            if (MdaUtils.isEmpty(bo.getCrdgrp().getRec().getSta())) {
                Argument<String> pathBox = new Argument<String>("", bo.getCrdgrp().getRec().getSta());
                Argument<String> pathBox1 = new Argument<String>("", bo.getCrdgrp().getRec().getErrmsg());
                checkValidity(pathBox, pathBox1, bo);

                bo.getCrdgrp().getRec().setSta(pathBox.value);
                bo.getCrdgrp().getRec().setErrmsg(pathBox1.value);

            }
            if (MdaUtils.compareTo(bo.getCrdgrp().getRec().getSta(), "W") == 0) {
//                Dynamic.sysWarningSet(SYSWRNTypeWarning, bo.getCrdgrp().getRec().getErrmsg(), "CLEARINGWARNINGS");
            } else {
//                Dynamic.sysWarningSet(SYSWRNTypeWarning, "", "CLEARINGWARNINGS");
            }
        } else {
            if (sysmodService.sptIsAnyLoaded(bo.getSpt().getSta())) {
                bo.setRcvobjtyp(bo.getCrdgrp().getRec().getRcvobjtyp());
            }
        }

    }

    /**
     * source:crtcri.0002.script
     */
    private void init950(CrtcriStoreBo crtcriStoreBo) {
        crtcriStoreBo.setClrsta("I");
        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        crtcriStoreBo.getCrdgrp().getRec().setOwnusr(extkey);
        defaultCrdgrpRecCustypN1000(crtcriStoreBo);
    }

    @Override
    public void beforeSave(CrtcriStoreBo transactionVo) {
        transactionVo.reset();
        transactionVo.reset();
        Crdgrp crdgrp = transactionVo.getCrdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(crdgrp.getRec());
        transactionServiceSet.setTransactionVo(transactionVo);
        crToolComponent.collectCbsMap(transactionServiceSet, "", crdgrp.getCbs(), "max");
        crToolComponent.collectPtsList(transactionServiceSet, crdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());

    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, CrtcriStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getCrdgrp().getRec().getInr());

    }

    public String getRef(CrtcriStoreBo bo) {
        bo.reset();
        if (Objects.isNull(SettleContext.getUserSession().getWrkbch()) || StringUtils.isEmpty(SettleContext.getUserSession().getWrkbch().getInr())) {
            SettleContext.getUserSession().setWrkbch(SettleContext.getUserSession().getBch());
        }
        Crdgrp crdgrp = bo.getCrdgrp();

        crToolComponent.getref(crdgrp);
        return crdgrp.getRec().getOwnref();
    }


    /**
     * source:crtcri.0008.script
     *
     * @param trmTyp
     * @param bchInr
     * @param cur    bo.getCrdgrp().getCbs().getMax().getCur()
     * @return
     */
    public Act readClearingAct(String trmTyp, String bchInr, String cur) {
//        String sql = "select * from act WHERE HOLPTYINR = (SELECT PTYINR FROM BCH WHERE INR = '" + bchInr
//                + "') AND (CUR = ' ' OR CUR = '" + cur + "') AND TRMTYP = '"
//                + trmTyp + "' AND ACCTYP = 'O'";
        Bch bch = new Bch();
        bch.setInr(bchInr);
        Bch dbBch = bchMapper.selectByPrimaryKey(bch);
        if (Objects.isNull(dbBch)) {
            return new Act();
        }
        String sql = "select * from act WHERE HOLPTYINR  in (SELECT inr FROM pty WHERE extkey = '" + dbBch.getSwfcod()
                + "') AND (CUR = ' ' OR CUR = '" + cur + "') AND TRMTYP = '"
                + trmTyp + "' AND ACCTYP = 'O'";
        List<Act> actList = actMapper.dyncRead(ImmutableBiMap.of("sql", sql));
        if (CollectionUtils.isEmpty(actList) || actList.size() > 1) {
            return new Act();
        } else {
            Act rcvact = actList.get(0);
            int pos = Strings.pos(rcvact.getExtkey(), "A");
            if (pos > 0) {
//                text = Codetables.getCodeValueText(bo.getCrdgrp().getRec(), "rcvbchinr", bo.getCrdgrp().getRec().getRcvbchinr());
//                BRANCH || BCHNAME
                String text = dbBch.getBranch() + dbBch.getBchname();
                rcvact.setExtkey(Strings.mid(rcvact.getExtkey(), 1, pos - 1) +
                        Strings.mid(text, 1, 4) + Strings.mid(rcvact.getExtkey(), pos + 4));
            }
            pos = Strings.pos(rcvact.getExtkey(), "B");
            if (pos > 0) {
                rcvact.setExtkey(Strings.mid(rcvact.getExtkey(), 1, pos - 1)
                        + Strings.mid(Codetables.getCodetableLabelOfTd(cur, "CURKEY", Systems.getLang()), 1, 2)
                        + Strings.mid(rcvact.getExtkey(), pos + 2));
            }

            return rcvact;
        }
    }


    /**
     * source:crtcri.0000.script
     */
    public void checkValidity(Argument<String> rc, Argument<String> errMsg, CrtcriStoreBo bo) {
        rc.value = "";
        errMsg.value = "";
        // 检查是主报文还是副报文
        if (bo.getSmh().getGrpseq() == 2) {
            rc.value = "E";
            errMsg.value = "副报文无法进行清算，请选择对应的主报文进行清算";
            return;
        }
        String maxCur = bo.getCrdgrp().getCbs().getMax().getCur();
        // 读取发报行
        String sndKey = NULLSTR;
        if (Strings.len(bo.getSmh().getSndkey()) == 11) {
            sndKey = bo.getSmh().getSndkey();
        } else {
            sndKey = Strings.mid(bo.getSmh().getSndkey(), 1, 8) + Strings.mid(bo.getSmh().getSndkey(), 10);
        }
        //  DRF 多实体情况下，需要添加实体的判断
//        String sql = "SELECT INR FROM PTY WHERE EXTKEY = '" + sndKey + "' AND ETGEXTKEY= " + DpToolComponents.sqlLit(SettleContext.getUserSession().getEty().getEtg());
        Pty pty = ptyMapper.selectByExtkey(sndKey);
        if (Objects.nonNull(pty)) {
            ptsptaToolComponent.ptsmodGetPtsptaFromPtyINR(bo.getCrdgrp().getSnd(), pty.getInr());
            bo.getCrdgrp().getSnd().getPts().setRef(bo.getSmh().getF20());
        } else {
            rc.value = "E";
            errMsg.value = "找不到发报行或发报行记录不惟一，无法清算";
            return;
        }
        // 检查关联情况
        switch (bo.getSmh().getMsgtyp()) {
            case "103":
                // 判断是否带头寸的103报文
                // 1、非账户行发来的103，全部不带头寸
                // 2、账户行发来的103，53A、54A为空则带头寸，否则不带头寸
                if (!checkAccBank(bo)) {
                }
                break;
            case "202":
            case "910":
                // 判断是否是不带头寸的202/910报文
                if (!checkAccBank(bo)) {
                    rc.value = "E";
                    errMsg.value = "非账户行的202/910报文无法清算";
                    return;
                } else {
                    // 根据业务Reference判断
                    if (!MdaUtils.isEmpty(bo.getCrdgrp().getRec().getMsgref())) {
                        switch (Strings.mid(bo.getCrdgrp().getRec().getMsgref(), 6, 2)) {
                            case "FT":
                            case "PT":
                            case "FX":
                            case "DX":
                                // 资金类报文，不提示
                                rc.value = "E";
                                errMsg.value = "资金类头寸报文，请手工清分到资金部";
                                return;
                            case "LC":
                            case "AL":
                            case "AB":
                            case "SG":
                            case "AD":
                            case "CR":
                            case "LT":
                            case "BT":
                            case "BP":
                            case "OC":
                            case "CC":
                            case "CL":
                            case "CH":
                            case "IC":
                            case "LG":
                            case "GA":
                            case "GC":
                            case "CG":
                            case "EF":
                            case "IF":
                            case "OF":
                            case "FB":
                            case "FS":
                            case "FI":
                            case "JN":
                            case "CO":
                            case "KZ":
                            case "DD":
                            case "TZ":
                            case "JD":
                                // 非汇款业务
                                rc.value = "W";
                                errMsg.value = "未绑定业务报文，请确认后再提交";
                                break;
                            default:
                                if (MdaUtils.compareTo(Strings.mid(bo.getCrdgrp().getRec().getMsgref(), 1, 1), "J") == 0) {
                                    // 资金类报文，不提示
                                    rc.value = "E";
                                    errMsg.value = "资金类头寸报文，请手工清分到资金部";
                                    return;
                                }
                        }
                    } else {
                        // 认为是汇款业务
                        rc.value = "E";
                        errMsg.value = "请先绑定MT103报文后再进行清算";
                        return;
                    }
                }
                break;
            case "400":
            case "756":
                // 必须绑定头寸报文才能清算
                if (!checkAccBank(bo)) {
                    if (MdaUtils.isEmpty(bo.getCorsmh().getInr())) {
                        rc.value = "E";
                        errMsg.value = "非账户行的756/400请绑定头寸报文后再进行清算";
                        return;
                    }
                }
                break;
        }
        //  判断起息日
        /*TODO SMH  表没有valdat
        if (!MdaUtils.isEmpty(bo.getCorsmh().getInr())) {
            bo.getCrtp().setValdat(bo.getCorsmh().getValdat());
            if (Dates.diff(bo.getCorsmh().getValdat(), Dates.today()) > 0) {
                rc.value = "E";
                errMsg.value = "后起息的报文无法提前清分，请于" + Formats.format(bo.getCorsmh().getValdat(), "YYYY年MM月DD日") + "后进行清算";
                return;
            }
        } else {
            bo.getCrtp().setValdat(bo.getSmh().getValdat());
            if (Dates.diff(bo.getSmh().getValdat(), Dates.today()) > 0) {
                rc.value = "E";
                errMsg.value = "后起息的报文无法提前清分，请于" + Formats.format(bo.getSmh().getValdat(), "YYYY年MM月DD日") + "后进行清算";
                return;
            }
        }*/
        // 读取同业账户（账户行情况下已经在判断是否账户行中读取）
        if (!MdaUtils.isEmpty(bo.getCorsmh().getInr()) && MdaUtils.isEmpty(bo.getSndact().getInr())) {
            // 读取账户行
            if (Strings.len(bo.getCorsmh().getSndkey()) == 11) {
                sndKey = bo.getCorsmh().getSndkey();
            } else {
                sndKey = Strings.mid(bo.getCorsmh().getSndkey(), 1, 8) + Strings.mid(bo.getCorsmh().getSndkey(), 10);
            }
            String sql = "SELECT INR FROM PTY WHERE EXTKEY = '" + sndKey + "'";
            List<Pty> ptyList = ptyMapper.dyncRead(ImmutableBiMap.of("sql", sql));

            if (CollectionUtils.isNotEmpty(ptyList) && ptyList.size() == 1) {
                ptsptaToolComponent.ptsmodGetPtsptaFromPtyINR(bo.getCrdgrp().getAcc(), ptyList.get(0).getInr());
                bo.getCrdgrp().getAcc().getPts().setRef(bo.getCorsmh().getF20());
            } else {
                rc.value = "W";
                errMsg.value = "找不到账户行或账户行记录不惟一，请手工指定发报行";
                return;
            }
//            readAct(bo.getSysmod().getBch().getPtyinr(), bo.getCrdgrp().getAcc().getPts().getPtyinr(), bo.getSndact());
            readAct("", bo.getCrdgrp().getAcc().getPts().getPtyinr(), bo.getSndact(), maxCur);
            if (Objects.isNull(bo.getSndact()) || StringUtils.isEmpty(bo.getSndact().getInr())) {
                rc.value = "E";
                errMsg.value = "账户行未维护存放同业账户，请在DBxACT中维护账号后清算";
                return;
            }
        }
        // 自动清分
        // 1、根据账号Tag59清分
        if (!MdaUtils.isEmpty(bo.getCrdgrp().getRec().getMsgact())) {
            List<Act> actListByExtkey = actMapper.getActListByExtkey(bo.getCrdgrp().getRec().getMsgact());
//            ctx.getDaoSession().dbRead(bo.getAplact(), bo.getCrdgrp().getRec().getMsgact(), "EXTKEY");
            if (CollectionUtils.isEmpty(actListByExtkey)) {
                rc.value = "W";
                errMsg.value = "客户账号在数据库中不存在，请手工清分";
                return;
            }
            bo.setAplact(actListByExtkey.get(0));
            // TODO act 表没有  bchkeyinr 考虑到多个PTA的情况
//            if (!MdaUtils.isEmpty(bo.getAplact().getBchkeyinr())) {
//                ctx.getDaoSession().dbRead(bo.getCrdgrp().getApl().getPta(), bo.getAplact().getBchkeyinr(), "BRANCHINR", bo.getAplact().getHolptyinr(), "PTYINR");
//            } else {
//                ctx.getDaoSession().dbRead(bo.getCrdgrp().getApl().getPta(), bo.getAplact().getHolptyinr(), "PTYINR");
//            }
            List<Pta> ptaList = ptaMapper.selectByPtyInr(bo.getAplact().getHolptyinr());
            if (CollectionUtils.isEmpty(ptaList)) {
                rc.value = "W";
                errMsg.value = "无此客户地址，请手工指定下划机构";
                MdaUtils.clear(bo.getCrdgrp().getApl().getPta());
                return;
            } else {
                if (ptaList.size() > 1) {
                    rc.value = "W";
                    errMsg.value = "此客户有多个地址，请手工指定下划机构";
                    MdaUtils.clear(bo.getCrdgrp().getApl().getPta());
                    return;
                }
            }
            bo.getCrdgrp().getApl().setPta(ptaList.get(0));
            PtsptaVo apl = ptsptaToolComponent.getPtsptaFromPtaINR(bo.getCrdgrp().getApl().getPta().getInr());
            bo.getCrdgrp().setApl(apl);
            bo.getCrdgrp().getRec().setRcvbchinr(bo.getCrdgrp().getApl().getPta().getBranchinr());
            // 有账号的报文只有MT103
            bo.getCrdgrp().getRec().setRcvobjtyp("CPD");
            // 判断对公汇款还是对私汇款
            // 台州银行判断客户号，第一位1-4为对私，5-8为对公
            switch (Strings.mid(bo.getCrdgrp().getApl().getPta().getPtyextkey(), 1, 1)) {
                case "1":
                case "2":
                case "3":
                case "4":
                    bo.getCrdgrp().getRec().setCustyp("1");
                    break;
                case "5":
                case "6":
                case "7":
                case "8":
                    bo.getCrdgrp().getRec().setCustyp("0");
                    break;
            }
        }
        // 2、根据OwnRef Tag21清分
        String branchInr = NULLSTR;
        String objTyp = NULLSTR;
        String objInr = NULLSTR;
        if (!MdaUtils.isEmpty(bo.getCrdgrp().getRec().getMsgref()) && MdaUtils.isEmpty(bo.getCrdgrp().getRec().getMsgact())) {
            String sql = "SELECT DISTINCT(BRANCHINR), OBJTYP, OBJINR FROM TRN WHERE OWNREF = '" + bo.getCrdgrp().getRec().getMsgref() + "'";
            List<Trn> trnList = trnMapper.dyncRead(ImmutableBiMap.of("sql", sql));
            if (CollectionUtils.isEmpty(trnList)) {
                rc.value = "W";
                errMsg.value = "国结系统中无对应此业务编号的交易，请手工指定下划机构";
                return;
            } else {
                branchInr = trnList.get(0).getBranchinr();
                objTyp = trnList.get(0).getObjtyp();
                objInr = trnList.get(0).getObjinr();
                bo.getCrdgrp().getRec().setRcvbchinr(branchInr);
                bo.getCrdgrp().getRec().setRcvobjtyp(objTyp);
                bo.getCrdgrp().getRec().setRcvobjinr(objInr);
            }
        }
        // 3、根据收报行的Swift Code清分
        String rcvKey = NULLSTR;
        if (MdaUtils.isEmpty(bo.getCrdgrp().getRec().getRcvbchinr())) {
            if (Strings.len(bo.getSmh().getRcvkey()) == 11) {
                rcvKey = bo.getSmh().getRcvkey();
            } else {
                rcvKey = Strings.mid(bo.getSmh().getRcvkey(), 1, 8) + Strings.mid(bo.getSmh().getRcvkey(), 10);
            }
            String sql = "SELECT INR FROM BCH WHERE SWFCOD = '" + rcvKey + "' AND LEV <> '0'";
            List<Bch> bchList = bchMapper.dyncRead(ImmutableBiMap.of("sql", sql));
            if (CollectionUtils.isNotEmpty(bchList) && bchList.size() == 1) {
                bo.getCrdgrp().getRec().setRcvbchinr(bchList.get(0).getInr());
            } else {
                rc.value = "W";
                errMsg.value = "多家分支行公用一个Swift Code，请手工指定下划机构";
                return;
            }
        }
        // 如果下发行没有权限经办业务，则转到它的上级经办机构   原代码的wrkflg 对应BCHTYP (1:经办行)
        String sql = "SELECT BCHTYP, BCHKEY FROM BCH WHERE INR = '" + bo.getCrdgrp().getRec().getRcvbchinr() + "'";
        String bchtyp = "";
        String bchKey = "";
        List<Bch> bchList = bchMapper.dyncRead(ImmutableBiMap.of("sql", sql));
        if (CollectionUtils.isNotEmpty(bchList)) {
            bchtyp = bchList.get(0).getBchtyp();
            bchKey = bchList.get(0).getBchkey();
        }
        String wrkInr = NULLSTR;
        if (MdaUtils.isEmpty(bchtyp) || MdaUtils.compareTo(bchtyp, "1") != 0) {
//            sql = "SELECT INR FROM BCH WHERE BRANCH = '" + bchKey + "'";
            sql = "SELECT INR FROM BCH WHERE bchtyp='1' and  BRANCH = '" + bchKey + "'";
            List<Bch> list = bchMapper.dyncRead(ImmutableBiMap.of("sql", sql));
            if (CollectionUtils.isNotEmpty(list)) {
                wrkInr = list.get(0).getInr();
            }
            bo.getCrdgrp().getRec().setRcvbchinr(wrkInr);
        }
        // 取得下发行的清算账号
        if (MdaUtils.compareTo(bo.getCrdgrp().getRec().getCustyp(), "1") == 0) {
            // 对私
            Act act = readClearingAct("SP-RIP", bo.getCrdgrp().getRec().getRcvbchinr(), maxCur);
            bo.setRcvact(act);
        } else {
            bo.getCrdgrp().getRec().setCustyp("0");
            // 对公
            Act act = readClearingAct("SP-RIC", bo.getCrdgrp().getRec().getRcvbchinr(), maxCur);
            bo.setRcvact(act);
        }
    }


    /**
     * source:crtcri.0003.script
     */
    public boolean checkAccBank(CrtcriStoreBo bo) {
       /* // 检查是否账户行关系
        // 账户行的账号持有者为清算中心
        // 需检查同业存放和存放同业
        boolean boolVar = false;
//        readAct(bo.getSysmod().getBch().getPtyinr(), bo.getCrdgrp().getSnd().getPts().getPtyinr(), bo.getSndact());
        readAct(SettleContext.getUserSession().getOwnpts().getPtyinr(), bo.getCrdgrp().getSnd().getPts().getPtyinr(),
                bo.getSndact(),bo.getCrdgrp().getCbs().getMax().getCur());
        if (ctx.getErrorCode() != tdOdbcNotFound) {
            if (!MdaUtils.isEmpty(bo.getSndact().getInr())) {
                boolVar = true;
            }
        } else {
            //检查同业存放
            readVOAct(bo.getCrdgrp().getSnd().getPts().getPtyinr(), bo.getSysmod().getBch().getPtyinr(), bo.getSndact());
            if (ctx.getErrorCode() != tdOdbcNotFound) {
                if (!MdaUtils.isEmpty(bo.getSndact().getInr())) {
                    boolVar = true;
                }
            }
        }*/
        return true;

    }

    /**
     * source:crtcri.0004.script
     *
     * @param holPtyInr
     * @param serPtyInr
     * @param act
     * @param cur       bo.getCrdgrp().getCbs().getMax().getCur()
     */
    public void readAct(String holPtyInr, String serPtyInr, Act act, String cur) {
        //todo  act  没有 SERPTYINR字段
//        String sql = "SELECT * from act WHERE HOLPTYINR = '" + holPtyInr + "' AND SERPTYINR = '" + serPtyInr + "' AND CUR = '" + cur + "' AND TYP = 'DB'";
        String sql = "SELECT * from act WHERE HOLPTYINR = '" + holPtyInr + "' AND CUR = '" + cur + "' AND TYP = 'DB'";
        List<Act> actList = actMapper.dyncRead(ImmutableBiMap.of("sql", sql));
        if (CollectionUtils.isNotEmpty(actList)) {
            BeanUtil.copyProperties(actList.get(0), act);
        }

    }


    /**
     * source:crtcri.0020.script
     */
    public void manualClear(CrtcriStoreBo bo) {
        bo.setCrtspt(new Spt());
        bo.getCrtspt().setObjtyp(bo.getCrdgrp().getRec().getRcvobjtyp());
        bo.getCrtspt().setObjinr(bo.getCrdgrp().getRec().getRcvobjinr());
        switch (bo.getCrtspt().getObjtyp()) {
            case "CPD":
                bo.getCrtspt().setFrm("CPTADV");
                break;
            case "BOD":
                bo.getCrtspt().setFrm("BOTSET");
                break;
            case "CCD":
                bo.getCrtspt().setFrm("CCTSET");
                break;
            case "BED":
                bo.getCrtspt().setFrm("BETSET");
                break;
            case "BTD":
                bo.getCrtspt().setFrm("BTTSET");
                break;
            case "GID":
                bo.getCrtspt().setFrm("GETSET");
                break;
            case "CLD":
                bo.getCrtspt().setFrm("CLTSET");
                break;
        }
        bo.getCrtspt().setTxt("N-SWIFT Doc Clearing");
        bo.getCrtspt().setObjnam(Strings.getText(MdaUtils.getI18NString("crtcri", "CT000008"), bo.getCrdgrp().getRec().getNam()));
        bo.getCrtspt().setOwnusr(SettleContext.getUserSession().getUsr().getExtkey());
        bo.getCrtspt().setOwnusg("OU");
        bo.getCrtspt().setCreobjtyp("TRN");
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();
        String trninr = transactionHelpService.getTrninr();
        bo.getCrtspt().setCreobjinr(trninr);
        // 业务的经办机构
//        bo.getCrtspt().setBchkeyinr(Datas.getContent(this, bo.getCrdgrp().getRec().getRcvobjtyp() + "GRP\\REC\\BRANCHINR"));
        String bchkeyinr = Utils.getFieldValue(bo, bo.getCrdgrp().getRec().getRcvobjtyp() + "GRP\\REC\\BRANCHINR");
        bo.getCrtspt().setBchkeyinr(bchkeyinr);
        if (MdaUtils.isEmpty(bo.getCrtspt().getBchkeyinr())) {
            bo.getCrtspt().setBchkeyinr(bo.getCrdgrp().getRec().getRcvbchinr());
        }
        bo.getCrtspt().setBranchinr(bo.getCrtspt().getBchkeyinr());
        bo.getCrtspt().setCredat(Dates.today());
        bo.getCrtspt().setOrdinr(ordmodService.ordGetINR(trnmodVo.getOrdmod()));
        //清算发起标识 TODO
//        bo.getCrtspt().setClrflg("Y");
        bo.getCrtspt().setHldflg("R");
        IStream clrStm = new StreamImpl();
        Streams.streamClear(clrStm);
//        Systems.setContext("SAVEDATASKIPEMPTY", true);
//        IModule mod = MdaDriver.getModule(this, bo.getCrdgrp().getRec().getRcvobjtyp() + "GRP");
        Object mod = Utils.getFieldValue(bo, bo.getCrdgrp().getRec().getRcvobjtyp() + "GRP");
        Utils.saveData(mod, clrStm, "", bo.getCrdgrp().getRec().getRcvobjtyp() + "GRP\\");
//        Systems.setContext("SAVEDATASKIPEMPTY", false);
        // sta - ?
        String sta = "CLI";
        sysmodService.sptSaveEntry(bo.getCrtspt(), Utils.getFieldValue(mod, "REC"), sta, clrStm, "");

    }


    /**
     * source:crtcri.0021.script
     */
    public void defaultCrdgrpRecCustypN1000(CrtcriStoreBo bo) {

        if (MdaUtils.compareTo(bo.getCrdgrp().getRec().getRcvobjtyp(), "CPD") != 0) {
            bo.getCrdgrp().getRec().setCustyp("0");
        }
    }

    /**
     * source:crtcri.0016.script
     */
    public void defaultCrdgrpRecNamN1000(CrtcriStoreBo bo) {
        //! Default name of contract; usually contract amount and information about the client is used
        if (!bo.getCrdgrp().getRec().isModified("nam")) {
            if (MdaUtils.compareTo(bo.getCrdgrp().getRec().getClrtyp(), "O") == 0) {
//                Dynamic.ptsmodGetPtyOfPtspta(bo.getCrdgrp().getRcv(), bo.getTrnmod().getPtsmod().getPty());
                //根据前一个参数里的pta的ptyinr 读取出pty 放到 ptsmod的pty下
//                Pty dbPty = ptyCacheService.getPtyById(bo.getCrdgrp().getRcv().getPta().getPtyinr());
                String nam = bo.getCrdgrp().getRec().getMsgtyp() + " " + bo.getCrdgrp().getCbs().getMax().getCur() + " "
                        + Formats.fmtAmount(bo.getCrdgrp().getCbs().getMax().getAmt(), bo.getCrdgrp().getCbs().getMax().getCur())
                        + " " + bo.getCrdgrp().getRec().getRcvobjtyp() + "/" + bo.getCrdgrp().getRcv().getPts().getNam();
                bo.getCrdgrp().getRec().setNam(Strings.mid(nam, 1, 40));
            } else {
//                Dynamic.ptsmodGetPtyOfPtspta(bo.getCrdgrp().getSnd(), bo.getTrnmod().getPtsmod().getPty());
//                Pty dbPty = ptyCacheService.getPtyById(bo.getCrdgrp().getSnd().getPta().getPtyinr());

                String nam = bo.getCrdgrp().getRec().getMsgtyp() + " " + bo.getCrdgrp().getCbs().getMax().getCur() + " "
                        + Formats.fmtAmount(bo.getCrdgrp().getCbs().getMax().getAmt(), bo.getCrdgrp().getCbs().getMax().getCur())
                        + " " + bo.getCrdgrp().getRec().getRcvobjtyp() + "/" + bo.getCrdgrp().getSnd().getPta().getNam();
                bo.getCrdgrp().getRec().setNam(Strings.mid(nam, 1, 40));
            }
        }


    }

    public List<CodetableItem> getBranchCodeValue() {
        List<Map<String, Object>> list = crtpService.setBranchCodeValue();
        List<CodetableItem> collect = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)) {
            collect = list.stream().map(item -> {
                String val = (String) item.get("VAL");
                String txt = (String) item.get("TXT");
                return new CodetableItem(txt, val);
            }).collect(Collectors.toList());
        }
        return collect;
    }
}


