package com.brilliance.isc.funds.fttfcm.check;

import com.brilliance.isc.bo.Act;
import com.brilliance.isc.common.transaction.newcheck.AbstractModuleValidator;
import com.brilliance.isc.common.transaction.newcheck.ValidResult;
import com.brilliance.isc.common.transaction.newcheck.Validate;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.funds.bo.funds.FttfcmStoreBo;
import com.brilliance.isc.funds.bo.funds.FxtfcmStoreBo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.springframework.stereotype.Component;


import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.brilliance.mda.runtime.mda.Constants.*;

/**
 * @program: isc-funds
 * @ClassName: FttfcmValidator
 * @description: 结售汇平盘确认 Fttfcm 校验
 * @author: 肖桐
 * @create: 2024-10-11 11:28
 */
@Component
public class FttfcmValidator extends AbstractModuleValidator <FttfcmStoreBo>{

    @Override
    @PostConstruct
    public void register(){
        super.register();;
    }

    @Override
    public Map<String, String> validate(FttfcmStoreBo vo) {
        Map<String, String> maps = new HashMap<>();
        maps.putAll(super.validate(vo));
        return maps;
    }




    /**
     *   source:fttfcm.0018.script
     */
    @Validate(path="fttp.mt32m.s30x")
    public ValidResult checkFttpMt32mS30xN1000(FttfcmStoreBo bo){
        if( ( MdaUtils.compareTo(bo.getFttp().getMt32m().getS22b(),"CONF") == 0 || MdaUtils.compareTo(bo.getFttp().getMt32m().getS22b(),"ROLL") == 0 ) && MdaUtils.isEmpty(bo.getFttp().getMt32m().getS30x()) )
        {
            return  errorMandatory();
        }
        return null;
    }

    /**
     *   source:fttfcm.0022.script
     */
    @Validate(path="ftdgrp.cbs.ins.amt")
    public ValidResult checkFtdgrpCbsInsAmtN1000(FttfcmStoreBo bo){
         if( MdaUtils.compareTo(bo.getFttp().getPansta(),PanStaAdd) == 0 || MdaUtils.compareTo(bo.getFttp().getPansta(),PanStaEdit) == 0 )
        {
            if( MdaUtils.isEmpty(bo.getFtdgrp().getCbs().getIns().getAmt()) )
            {
                return  errorMandatory();
            }
        }


        return null;
    }

    /**
     *   source:fttfcm.0023.script
     */
    @Validate(path="ftdgrp.rec.cntfra")
    public ValidResult checkFtdgrpRecCntfraN1000(FttfcmStoreBo bo){
      if( MdaUtils.isEmpty(bo.getFtdgrp().getRec().getCntfra()) )
        {
            return  errorMandatory();
        }


        return null;
    }

    /**
     *   source:fttfcm.0024.script
     */
    @Validate(path="ftdgrp.rec.matdat")
    public ValidResult checkFtdgrpRecMatdatN1000(FttfcmStoreBo bo){
        if( MdaUtils.isEmpty(bo.getFtdgrp().getRec().getMatdat()) )
        {
            return  errorMandatory();
        }
        else
        {
            if( ! MdaUtils.isEmpty(bo.getFtdgrp().getRec().getValdat()) && Dates.diff(bo.getFtdgrp().getRec().getValdat(),bo.getFtdgrp().getRec().getMatdat())>0 )
            {
                return ValidResult.build(false, MdaUtils.getI18NString("fttfcm","CT000008"));

                }
        }


        return null;
    }

    /**
     *   source:fttfcm.0025.script
     */
    @Validate(path="ftdgrp.rec.rat")
    public ValidResult checkFtdgrpRecRatN1000(FttfcmStoreBo bo){
        if( MdaUtils.isEmpty(bo.getFtdgrp().getRec().getRat()) )
        {
            return  errorMandatory();
        }


        return null;
    }

    /**
     *   source:fttfcm.0038.script
     */
    @Validate(path="ftdgrp.rec.bnktyp")
    public ValidResult checkFtdgrpRecBnktypN1000(FttfcmStoreBo bo){
        if( MdaUtils.isEmpty(bo.getFtdgrp().getRec().getBnktyp()) )
        {
            return  errorMandatory();
        }


        return null;
    }




}
